%global debug_package %{nil}

# for kubernetes build
%global _buildshell  /bin/bash
%global _checkshell  /bin/bash

%global goipath         github.com/kubernetes/kubernetes
%global commit          95ee5ab382d64cfe6c28967f36b53970b8374491
%global shortcommit     %(c=%{commit}; echo ${c:0:7})

Summary:        Container cluster management
Name:           kubernetes
Version:        1.24.4
Release:        4%{?dist}
License:        ASL 2.0
URL:            https://k8s.io/kubernetes
Source0:        https://%{goipath}/archive/%{commit}/kubernetes-%{shortcommit}.tar.gz
Source1:        kube-proxy.service
Source2:        kube-apiserver.service
Source3:        kube-scheduler.service
Source4:        kube-controller-manager.service
Source5:        kubelet.service
Source6:        environ-apiserver
Source7:        environ-config
Source8:        environ-controller-manager
Source9:        environ-kubelet
Source10:       environ-kubelet.kubeconfig
Source11:       environ-proxy
Source12:       environ-scheduler
Source13:       kubernetes-accounting.conf
Source14:       kubeadm.conf
Source15:       kubernetes.conf
Source16:       %{name}.sysusers

Requires: kubernetes-master = %{version}-%{release}
Requires: kubernetes-node = %{version}-%{release}

%description
Container cluster management

%package master
Summary: Kubernetes services for control plane host
BuildRequires: golang systemd rsync go-md2man go-bindata

Requires(pre): shadow-utils
Requires: kubernetes-client = %{version}-%{release}

%description master
Kubernetes services for control plane host

%package node
Summary: Kubernetes services for worker node host
BuildRequires: golang systemd rsync go-md2man go-bindata

Requires: (containerd or cri-o)
Requires: conntrack-tools
Requires: socat
Requires: kubernetes-client = %{version}-%{release}
Requires(pre): shadow-utils
Suggests: containerd

%description node
Kubernetes services for worker node host

%package  kubeadm
Summary:  Kubernetes tool for standing up clusters
Requires: kubernetes-node = %{version}-%{release}
Requires: containernetworking-plugins cri-tools

%description kubeadm
Kubernetes tool for standing up clusters

%package client
Summary: Kubernetes client tools
BuildRequires: golang go-bindata make

%description client
Kubernetes client tools like kubectl

%prep
%autosetup -n kubernetes-%{commit} -p1

mkdir -p src/k8s.io/kubernetes
mv $(ls | grep -v "^src$") src/k8s.io/kubernetes/.

%build
pushd src/k8s.io/kubernetes/
source hack/lib/init.sh
kube::golang::setup_env

export KUBE_GIT_TREE_STATE="clean"
export KUBE_GIT_COMMIT=%{commit}
export KUBE_GIT_VERSION=v{version}
export KUBE_EXTRA_GOPATH=$(pwd)/Godeps/_workspace

make WHAT="cmd/kube-proxy"
make WHAT="cmd/kube-apiserver"
make WHAT="cmd/kube-controller-manager"
make WHAT="cmd/kubelet"
make WHAT="cmd/kubeadm"
make WHAT="cmd/kube-scheduler"
make WHAT="cmd/kubectl"
make WHAT="cmd/gendocs"
make WHAT="cmd/genkubedocs"
make WHAT="cmd/genman"
make WHAT="cmd/genyaml"
kube::util::gen-docs .

%install
pushd src/k8s.io/kubernetes/
source hack/lib/init.sh
kube::golang::setup_env

%ifarch ppc64le
output_path="_output/local/go/bin"
%else
output_path="${KUBE_OUTPUT_BINPATH}/$(kube::golang::host_platform)"
%endif

install -m 755 -d %{buildroot}%{_bindir}
for bin_name in kube-proxy kube-apiserver kube-controller-manager \
                kubelet kubeadm kube-scheduler kubectl; do
    install -pm 755 ${output_path}/$bin_name %{buildroot}%{_bindir}/
done

install -Dpm 0644 %{SOURCE14} -t %{buildroot}/%{_sysconfdir}/systemd/system/kubelet.service.d

install -d -m 0755 %{buildroot}%{_datadir}/bash-completion/completions/
%{buildroot}%{_bindir}/kubectl completion bash > %{buildroot}%{_datadir}/bash-completion/completions/kubectl
install -d -m 0755 %{buildroot}%{_datadir}/zsh-completion/completions/
%{buildroot}%{_bindir}/kubectl completion zsh > %{buildroot}%{_datadir}/zsh-completion/completions/kubectl
install -d -m 0755 %{buildroot}%{_datadir}/fish-completion/completions/
%{buildroot}%{_bindir}/kubectl completion fish > %{buildroot}%{_datadir}/fish-completion/completions/kubectl

install -d -m 0755 %{buildroot}%{_sysconfdir}/%{name}
install -d -m 0700 %{buildroot}%{_sysconfdir}/%{name}/manifests

install -m 644 -T %{SOURCE6} %{buildroot}%{_sysconfdir}/%{name}/apiserver
install -m 644 -T %{SOURCE7} %{buildroot}%{_sysconfdir}/%{name}/config
install -m 644 -T %{SOURCE8} %{buildroot}%{_sysconfdir}/%{name}/controller-manager
install -m 644 -T %{SOURCE9} %{buildroot}%{_sysconfdir}/%{name}/kubelet
install -m 644 -T %{SOURCE10} %{buildroot}%{_sysconfdir}/%{name}/kubelet.kubeconfig
install -m 644 -T %{SOURCE11} %{buildroot}%{_sysconfdir}/%{name}/proxy
install -m 644 -T %{SOURCE12} %{buildroot}%{_sysconfdir}/%{name}/scheduler

install -Dpm 0644 %{SOURCE15} -t %{buildroot}/%{_tmpfilesdir}
install -D -m644 -vp %{SOURCE16} %{buildroot}%{_sysusersdir}/%{name}.conf
install -Dpm 0644 %{SOURCE13} -t %{buildroot}/%{_sysconfdir}/systemd/system.conf.d

install -d -m 0755 %{buildroot}%{_unitdir}
for src in %{SOURCE1} %{SOURCE2} %{SOURCE3} %{SOURCE4} %{SOURCE5}; do
    install -m 0644 $src -t %{buildroot}%{_unitdir}/
done

install -Dpm 644 docs/man/man1/*.1 -t %{buildroot}%{_mandir}/man1

install -d %{buildroot}%{_sharedstatedir}/kubelet

mkdir -p %{buildroot}/run
install -d -m 0755 %{buildroot}/run/%{name}/
popd

mv src/k8s.io/kubernetes/CHANGELOG/CHANGELOG-*.md .
mv src/k8s.io/kubernetes/*.md .
mv src/k8s.io/kubernetes/LICENSE .
rm CHANGELOG.md

%check

%pre master
%sysusers_create_compat %{SOURCE16}

%post master
%systemd_post kube-apiserver kube-scheduler kube-controller-manager

%preun master
%systemd_preun kube-apiserver kube-scheduler kube-controller-manager

%postun master
%systemd_postun kube-apiserver kube-scheduler kube-controller-manager


%pre node
%sysusers_create_compat %{SOURCE16}

%post node
%systemd_post kubelet kube-proxy
if [[ `systemctl show kubelet | grep -q -e CPUAccounting=no -e MemoryAccounting=no; echo $?` -eq 0 ]]; then
  systemctl daemon-reexec
fi

%preun node
%systemd_preun kubelet kube-proxy

%postun node
%systemd_postun kubelet kube-proxy

%files
# blank pkg as it depends on master and node

%files master
%license LICENSE
%doc *.md
%dir %{_sysconfdir}/%{name}
%config(noreplace) %{_sysconfdir}/%{name}/apiserver
%config(noreplace) %{_sysconfdir}/%{name}/scheduler
%config(noreplace) %{_sysconfdir}/%{name}/config
%config(noreplace) %{_sysconfdir}/%{name}/controller-manager
%{_sysusersdir}/%{name}.conf
%{_tmpfilesdir}/kubernetes.conf
%attr(754, -, kube) %caps(cap_net_bind_service=ep) %{_bindir}/kube-apiserver
%{_bindir}/kube-controller-manager
%{_bindir}/kube-scheduler
%{_unitdir}/kube-apiserver.service
%{_unitdir}/kube-controller-manager.service
%{_unitdir}/kube-scheduler.service
%verify(not size mtime md5) %attr(755, kube,kube) %dir /run/%{name}
%{_mandir}/man1/kube-apiserver.1*
%{_mandir}/man1/kube-controller-manager.1*
%{_mandir}/man1/kube-scheduler.1*

%files node
%license LICENSE
%doc *.md
%dir %{_sysconfdir}/%{name}
%dir %{_sysconfdir}/%{name}/manifests
%config(noreplace) %{_sysconfdir}/%{name}/config
%config(noreplace) %{_sysconfdir}/%{name}/kubelet
%config(noreplace) %{_sysconfdir}/%{name}/proxy
%config(noreplace) %{_sysconfdir}/%{name}/kubelet.kubeconfig
%config(noreplace) %{_sysconfdir}/systemd/system.conf.d/kubernetes-accounting.conf
%{_tmpfilesdir}/kubernetes.conf
%{_sysusersdir}/%{name}.conf
%{_bindir}/kubelet
%{_bindir}/kube-proxy
%{_unitdir}/kube-proxy.service
%{_unitdir}/kubelet.service
%dir %{_sharedstatedir}/kubelet
%verify(not size mtime md5) %attr(755, kube,kube) %dir /run/%{name}
%{_mandir}/man1/kubelet.1*
%{_mandir}/man1/kube-proxy.1*

%files kubeadm
%license LICENSE
%doc *.md
%{_bindir}/kubeadm
%dir %{_sysconfdir}/systemd/system/kubelet.service.d
%config(noreplace) %{_sysconfdir}/systemd/system/kubelet.service.d/kubeadm.conf
%{_mandir}/man1/kubeadm.1*
%{_mandir}/man1/kubeadm-*

%files client
%license LICENSE
%doc *.md
%{_bindir}/kubectl
%{_datadir}/bash-completion/completions/kubectl
%{_datadir}/zsh-completion/completions/kubectl
%{_datadir}/fish-completion/completions/kubectl
%{_mandir}/man1/kubectl.1*
%{_mandir}/man1/kubectl-*

%changelog
* Fri Apr 28 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 1.24.4-4
- Rebuilt for OpenCloudOS Stream 23.05

* Fri Mar 31 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 1.24.4-3
- Rebuilt for OpenCloudOS Stream 23

* Tue Dec 20 2022 Shuo Wang <abushwang@tencent.com> - 1.24.4-2
- remove unused option for kube-apiserver.service

* Fri Dec 16 2022 Shuo Wang <abushwang@tencent.com> - 1.24.4-1
- initial build
